/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_16
**
**  editor routines
*/
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <ssdef.h>
#include <smgdef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define EDITOR_SIZE 40

globalref int captive;

char *stuff[] =	{
    "DIRECTORY MANAGER  -  EDITOR OPTIONS",
    "",
    "",
    "",
    "",
    "You have not defined an editor via ^D, so you may",
    "now select either EDT, TPU or the editor of your choice",
    "",
    "",
    "[H]elp",
    "[E]dt ",
    "[T]pu ",
    "[O]ther",
    "",
    "Select Option [Return to previous screen]",
    "",
    "",
    0};

edit_file(dir,filename)
char *dir;
char *filename;
{
    int i, proc_id, status, length, screen;
    char editor[EDITOR_SIZE+1];
    char trans[100];

    /*
    **  confirm intent to Edit
    */
    sprintf(junk,"Edit %s Y/N [N]",filename);
    if (!confirm_user(junk,"Y"))
        return(FALSE);

    /*
    **  determine editor and call it
    */
    if (editor_defn[0])
    {
        strcpy(trans,editor_defn);

        for (i = 0; i < strlen(trans); i++)
            trans[i] = toupper(trans[i]);

        smg$save_physical_screen(&paste,&screen);
        if (strncmp(trans,"EDT",3) == 0)
            edit_edt_file(filename,trans);
        else if (strncmp(trans,"TPU",3) == 0)
            edit_tpu_file(filename,trans);
        else if (strlen(trans))
            edit_other_file(filename,trans);
        smg$restore_physical_screen(&paste,&screen);
 
        return(TRUE);
    }
    
    smg$erase_display(&display_dcl);
    smg$set_cursor_abs(&display_dcl,&1,&1);

    for (i = 0; stuff[i] != 0; i++)
    {
        length = strlen(stuff[i]);
        sprintf(junk,"%*.*s%s",40-(length/2),40-(length/2),"",stuff[i]);
        smg$put_line(&display_dcl,DSC1(junk),&1,0);
    }
    smg$paste_virtual_display(&display_dcl,&paste,&1,&1);

    while(1)
    {
        switch(get_key(NO_AUTO))
        {
    case SMG$K_TRM_UPPERCASE_H:
    case SMG$K_TRM_LOWERCASE_H:
            smg$save_physical_screen(&paste,&screen);
            swing_help("SWING COMMANDS FILE EDIT");
            smg$restore_physical_screen(&paste,&screen);
    	    break;
    
    case SMG$K_TRM_UPPERCASE_E:
    case SMG$K_TRM_LOWERCASE_E:
            smg$erase_display(&display_dcl);
            smg$save_physical_screen(&paste,&screen);
            edit_edt_file(filename,"EDT");
            smg$restore_physical_screen(&paste,&screen);
            goto leave_me;
                
    case SMG$K_TRM_UPPERCASE_T:
    case SMG$K_TRM_LOWERCASE_T:
            smg$erase_display(&display_dcl);
            smg$save_physical_screen(&paste,&screen);
            edit_tpu_file(filename,"TPU");
            smg$restore_physical_screen(&paste,&screen);
            goto leave_me;
    
    case SMG$K_TRM_UPPERCASE_O:
    case SMG$K_TRM_LOWERCASE_O:
            /*
            **  get user's editor
            */
            length = 0;
            editor[0] = 0;
	    if (captive)
	    {
	        smg$put_line(&display_dcl,DSC1(CAPTIVE_ERRMSG),&0,0);
		break;
	    }
            smg$read_string(&keyboard,
                            DSCA1(editor),
                            DSC2("Input name of editor to be used [None] ? "),
                            &EDITOR_SIZE,0,0,0,&length,0,&display_dcl);
    
            if (length == 0 || editor[0] == 0)
                break;
    
            editor[length] = 0;
    
            smg$erase_display(&display_dcl);
            smg$save_physical_screen(&paste,&screen);
            edit_other_file(filename,editor);
            smg$restore_physical_screen(&paste,&screen);
            goto leave_me;
            break;

    default:
            goto leave_me;

        }
    }
leave_me:;
    smg$unpaste_virtual_display(&display_dcl, &paste);
}

edit_exception(sigarr, mecharr)
int *sigarr, *mecharr;
{
    /*
    **  unwind and return
    */
    lib$sig_to_ret(sigarr, mecharr);

    return(SS$_UNWIND);
}

edit_edt_file(filename,trans)
char *filename;
char *trans;
{
    int flags;

    VAXC$ESTABLISH(edit_exception);

    flags = 0;

#ifdef __DECC /*** Use spawn if DECC ***/
    if (strlen(trans) > 4 && access(&trans[4],0) == 0)
    {
        sprintf(junk,"EDITX/%s %s",trans,filename);
    }
    else if (access("sys$login:edtini.edt",0) == 0)
    {
        sprintf(junk,"EDITX/EDT/COMMAND=sys$login:edtini.edt %s", filename);
    }
    else
    {
        sprintf(junk,"EDITX/EDT %s",filename);
    }
    spawn_process(junk,0);
#else /*** Use callable EDT if VAXC ***/
    if (strlen(trans) > 4 && access(&trans[4],0) == 0)
    {
        edt$edit(DSC1(filename),0,DSC2(&trans[4]),0,&flags);
    }
    else if (access("sys$login:edtini.edt",0) == 0)
    {
        edt$edit(DSC1(filename),0,DSC2("sys$login:edtini.edt"),0,&flags);
    }
    else
        edt$edit(DSC1(filename),0,0,0,&flags);
#endif
}

edit_tpu_file(filename,trans)
char *filename;
char *trans;
{
    VAXC$ESTABLISH(edit_exception);

    sprintf(junk,"%s %s",trans,filename);
    tpu$tpu(DSC1(junk));

}

edit_other_file(filename,trans)
char *filename;
char *trans;
{
    VAXC$ESTABLISH(edit_exception);

    sprintf(junk,"%s %s",trans,filename);
    spawn_process(junk,0);
}
