/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_24.c  - allocated / control dstructures
*/
#include <stdio.h>
#include <ctype.h>
#include <lnmdef.h>
#include <descrip.h>
#include <ssdef.h>
#include <psldef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

static char first_translation[SPEC_SIZE+1];

#define TRNLNM_STRING_SIZE 	255
#define MAX_TRANSLATIONS	20
/*
**  unlink the dstructure structure p
*/
extract_structure(p)
DSTRUCTURE *p;
{
    DSTRUCTURE *q;

    /*
    **  save if necessary
    */
    if (p->to_be_saved)
        save_nodes(p,TRUE);

    /*
    **  free up the nodes
    */
    free_nodes(p->fnp);

    /*
    **  if its the first structure in the list
    */
    if (fdp == p)
    {
        fdp = cdp = p->forw_ptr;
        free(p);
    }
    else
    {
        /*
        **  find structure pointing to this structure
        */
        q = fdp;

        while(q->forw_ptr != p)
        {
            q = q->forw_ptr;
        }
       
        /*
        **  unlink
        */
        q->forw_ptr = p->forw_ptr;
        free(p);

        cdp = q;
    }

    set_default(cdp->fnp->dir_name);

    /*
    **  repair the display
    */
    load_lines();

    /*
    **  find where we start
    */
    set_init_position();

    /*
    **  update display
    */
    repair_display();

    /*
    **  done
    */
    return(TRUE);
}

add_directory_structure(directory)
char *directory;
{
    int i, j, status;
    DSTRUCTURE *tmp;
    char cur_dir[SPEC_SIZE+1];
    char dir[SPEC_SIZE+1];
    char name[SPEC_SIZE+1];
    char device[SPEC_SIZE+1];
    char cmndir[SPEC_SIZE+1];
    
    /*
    **  get the root spec for this node
    */
    get_root_dir(directory,dir,name);

    /*
    **  call set_def to make sure root exists
    */
    if (!set_default(dir))
        return(FALSE);

    /*
    **  pick up the current device in case it wasn't specified
    */
    show_def(cur_dir);

    /* 
    ** if device is SYS$SYSROOT, make it SYS$SPECIFIC, 
    ** set up cmndir for SYS$COMMON, and set sysroot_flag
    */
    if (strstr(cur_dir,"SYS$SYSROOT")) {
	strcpy(junk,"SYS$SPECIFIC:");
	i = 0;
	while (directory[i++] != '[');
        strcat(junk,&directory[i-1]);
	strcpy(directory,junk);
	strcpy(cmndir,"SYS$COMMON:");
        strcat(cmndir,&directory[13]);
	get_root_dir(directory,dir,name);
	set_default(dir);
	show_def(cur_dir);
	set_default(directory);
	sysroot_flag = 1;
   }

    /*
    **  get the root spec for this node, this time with device if device
    **  wasn't specified last time
    */
    get_root_dir(cur_dir,dir,name);

    /*
    **  if this is the first translation we've made we'll record it as such
    */
    if (!first_translation[0])
	strcpy(first_translation,directory);

    tmp = fdp;
    while(tmp)
    {
        if (strcmp(tmp->dir_name,dir) == 0)
        {
            /*
            **  it's there already, don't create
            */
            sprintf(junk,"Directory structure %s already loaded",dir);
            warning(junk);
	    if(sysroot_flag == 1)
	        goto add_sys$common;
            return(TRUE);
        }
        tmp = tmp->forw_ptr;
    }

    /*
    **  now grab a structure 
    */
    cdp = grab_dstructure();

    /*
    **  get root dir info for structure header
    */
    strcpy(cdp->dir_name,dir);
    strcpy(cdp->name,name);
    
    /*
    **  strip away the directories
    */
    show_def(device);
    device[strcspn(device,":") + 1] = 0;
    strcpy(cdp->device,device);
    construct_file_specs(cdp->save_file_home,
                         cdp->save_file_away,
                         cdp->save_file_ss);

    /*
    **  now let's see if there's a swing save file we can use
    */
    if (!load_nodes_from_file(cdp))
    {
        /*
        **  no - we'll have to search out the node
        **  examine the structure
        */
        if (!search_nodes(cdp->dir_name,cdp->name,cdp))
        {
            free(cdp);
            return(FALSE);
        }
    }

    /*
    **  fix first_translation if load_nodes_from_file changed root
    */
    if (!strstr(first_translation,"[000000") &&
         strstr(cdp->dir_name,"[000000")) {
	i=0;j=0;
	while (strncmp(&first_translation[i],"[",1))
	    junk[j++] = first_translation[i++];
	i += 1;
	strcpy(&junk[j],"[000000.");
	j += 8;
	strcpy(&junk[j],&first_translation[i]);
	strcpy(first_translation,junk);
    }

    /*
    **  link it in
    */
    tmp = fdp;
    if (tmp)
    {
        while(tmp->forw_ptr)
        {
            tmp = tmp->forw_ptr;
        }
    }

    if (!tmp)
        fdp = cdp;
    else if (tmp)            
    {
        tmp->forw_ptr = cdp;
    }

    /*
    **  now build up a new set of line pointers
    */
    load_lines();

add_sys$common:
    /*
    **  if node was in SYS$SYSROOT and sysroot_flag is set to 1,
    **  add node for SYS$COMMON and increment sysroot_flag to
    **	ensure we won't try to do this twice
    */
    if (sysroot_flag == 1) {
        sysroot_flag = 2;
	if(set_default(cmndir))
	   first_translation[0] = 0;
        add_directory_structure(cmndir);
    }
    return(TRUE);
}

load_lines()
{
    int line = 0;
    int level = 0;
    LINE *lptr;
    DSTRUCTURE *dptr;

    /*
    **  free existing storage
    */
    if (first_line_ptr)
        free_lines(first_line_ptr);

    total_lines = 0;

    /*
    **  walk down the dstructures
    */
    dptr = fdp;
    while(dptr)
    {
        lptr = grab_line();
        load_line(lptr,total_lines - 1,level,dptr->fnp,dptr);
        load_up_node_names(dptr);
        if (dptr->forw_ptr)
        {
            /*
            **  throw in a blank line
            */
            lptr = grab_line();
            lptr->dp = 0;
        }
        dptr = dptr->forw_ptr;
    }
    return(TRUE);
}

grab_dstructure()
{
    DSTRUCTURE *p;

    if (p = calloc(1, sizeof(DSTRUCTURE)))
    {
        return(p);
    }

    panic(1,"cannot allocate any more memory","grab_dstructure");
}

/*
**  we must take care - user can specify a dir or dev that uses a search list,
**  good example is sys$manager
**
**  must find all the associated roots and add 'em all in
*/
add_a_dstructure(specification)
char *specification;
{
    int added;

    first_translation[0] = 0;

    added = add_dstructure(specification);

    if (added)
    {
        /*
        **  call set_def to see if we can go to the desired directory
        */
        set_default(first_translation);

        /*
        **  find where we start
        */
        set_init_position();
        
        /*
        **  repair display
        */
        refresh_display();
    }

    return(added);

}

add_dstructure(specification)
char *specification;
{
    long result,added;
    int status,attr,index,index_rla,attrib,attrib_rla,i,j;
    unsigned char acmode;
    char res[TRNLNM_STRING_SIZE];
    char head[TRNLNM_STRING_SIZE],tail[TRNLNM_STRING_SIZE];
    char trans[MAX_TRANSLATIONS][TRNLNM_STRING_SIZE];

    /*
    **  $trnlnm item list
    */
    struct {
        short buffer_length;
        short item_code;
        long  buffer_address;
        long  return_length_address;
     } itmlst[4] = {	
			4,	LNM$_INDEX,	&index,	&index_rla,
			100,	LNM$_STRING,	res,	&result,
			4,	LNM$_ATTRIBUTES,&attrib,&attrib_rla,
                        0,	0,		0,	0};

    /*
    **  a few vars for $trnlnm
    */
    attr = LNM$M_CASE_BLIND;
    acmode = PSL$C_USER;

    /*
    **  structures added so far = 0
    */
    added = 0;

    /*
    **  now chop the supplied specification up into a head and tail - 
    **  if log_nam then head = log_nam, if adir then tail = dir
    */
    tail[0] = head[0] = 0;
    strcpy(head,specification);

    for (i = 0; i < strlen(head); i++)
    {

	/*  trim off any trailing colon  */
         if (head[i] == ':')
         {
              head[i] = 0;
              break;
         }

	/*  head stops at the directory brace  */
         if (head[i] == '[')
         {
             head[i] = 0;
             break;
        }
    }

    /*
    **  if we've got a directory brace, then put it and all
    **  after it into the tail
    */
    for (i = 0; i < strlen(specification); i++)
    {
         if (specification[i] == '[')
             strcpy(tail,&specification[i]);
    }

    /*
    **  if no head (just a tail) nothing to translate
    */
    if (!strlen(head))
    {
        if (add_directory_structure(tail))
            added++;
    }
    else
    {
        /*
        **  pull out all the translations, reiterating as necessary
        */
        result = 0;
        for (index = 0; index < MAX_TRANSLATIONS; index++) 
        {
	    res[0] = 0;
            status = sys$trnlnm(&attr ,
                                DSC1("LNM$FILE_DEV") ,
                                DSC2(head) ,
                                &acmode,
                                itmlst);

            /*
            **  if it worked and we got something
            */
            if (status == SS$_NORMAL && result && 
		!(attrib & LNM$M_CONCEALED) && !(attrib & LNM$M_TERMINAL))
            {
                res[result] = 0;
                strcpy(trans[index],res);
            }
            else 
            {
                break;
            }
        }

        /*
        **  now work on the translations - if we haven't got any we'll
        **  add in specification
        */

        for (i = 0; i < index; i++)
        {
            j = strlen(trans[i]);


	    /*
	    **  If the translation *ends* in '.]' or a colon...
            */
            if ((j > 2) && ((trans[i][j-2] == '.' && trans[i][j-1] == ']') ||
                            (trans[i][j-1] == ':')))
            {
                /*
                **  it's a disk, so don't use the translation
                */
                if (add_directory_structure(specification))
                    added++;
            }
            else
            {
                /*
                **  it's not a disk, so translate further
                */
		if (trans[i][strlen(trans[i])-1] != ':' &&
		    trans[i][strlen(trans[i])-1] != ']' )
		   strcat(trans[i],":");
                strcat(trans[i],tail);
                added += add_dstructure(trans[i]);
            }
        }

        if (index == 0)
        {
            /*
            **  try to add
            */
            if (add_directory_structure(specification))
                added++;
        }
    }
    return(added);
}
